/**
 * This cellular automata (CA) is a grid of cells, particularly its states. This
 * implementation is 1 dimensional.<br>
 * State can be <code>true</code>=1 or <code>false</code>=0.
 * 
 * @author TineL
 * @version 1.1, 2007-05-29
 * @since 1.0
 */
public class CA extends Abstract1DGrid {

  /** Array of cell states. */
  private boolean[] ca;

  /**
   * Creates <code>CA</code> with the specified size. All cell states are set
   * to 0 (<code>false</code>).
   * 
   * @param size the CA size.
   * @since 1.0
   */
  public CA(int size) {
    super(size);
    this.ca=new boolean[size];
  }

  /**
   * Creates <code>CA</code> with the given cell states.
   * 
   * @param states the cell states (<code>null</code> not permitted).
   * @since 1.0
   */
  public CA(boolean[] states) {
    super(states.length);
    this.ca=states;
  }

  /**
   * Generates a CA with random cell states considering the ratio of 1s that
   * occur in the CA.
   * 
   * @param size the CA size.
   * @param onesRatio the ratio of 1s that occur in the CA beside the 0s.
   * @return a random generated CA (<code>null</code> not possible).
   * @since 1.0
   */
  public static CA generateRandomCA(int size, double onesRatio) {
    CA ca=new CA(size);
    for (int i=0; i<size; i++) {
      ca.set(i, Math.random()<onesRatio?true:false);
    }
    return ca;
  }

  /**
   * @param index the cell index.
   * @return the cell state.
   * @since 1.0
   */
  public boolean get(int index) {
    return ca[index];
  }

  /**
   * Sets the state of the cell.
   * 
   * @param index the cell index.
   * @param state the cell new state.
   * @since 1.0
   */
  public void set(int index, boolean state) {
    ca[index]=state;
  }

  /**
   * @param index the cell index.
   * @return the state of the left cell.
   * @since 1.0
   */
  public boolean getLeft(int index) {
    return get(getLeftIndex(index));
  }

  /**
   * @param index the cell index.
   * @return the state of the right cell.
   * @since 1.0
   */
  public boolean getRight(int index) {
    return get(getRightIndex(index));
  }

  /**
   * @return the number of 1s (<code>true</code> states) in the CA.
   * @since 1.0
   */
  public int onesCount() {
    int ones=0;
    for (int i=0; i<size(); i++) {
      if (get(i)) ones++;
    }
    return ones;
  }

  /**
   * @return the ratio of 1s (<code>true</code> states) according to 0s in
   *         the CA.
   * @since 1.0
   */
  public double onesRatio() {
    return (double)onesCount()/size();
  }

  /**
   * @return <code>true</code> if CA has more or equal half 1s (<code>true</code>
   *         states) according to 0s.
   * @since 1.0
   */
  public boolean moreOnes() {
    return onesRatio()>=0.5;
  }

  /**
   * @return the number of 0s (<code>false</code> states) in the CA.
   * @since 1.0
   */
  public int zerosCount() {
    return size()-onesCount();
  }

  /**
   * @return the ratio of 0s (<code>false</code> states) according to 1s in
   *         the CA.
   * @since 1.0
   */
  public double zerosRatio() {
    return (double)zerosCount()/size();
  }

  /**
   * @return <code>true</code> if CA has more or equal half 0s (<code>false</code>
   *         states) according to 1s.
   * @since 1.0
   */
  public boolean moreZeros() {
    return zerosRatio()>=0.5;
  }

  /*
   * @since 1.0
   */
  public String toString() {
    String out="CA=[";
    for (int i=0; i<size(); i++) {
      out+=(get(i)==true)?"1":" ";
    }
    out+="]";
    return out;
  }
}
